@@ -20,14 +20,7 @@ import datetime
 import decimal
 import unittest
 
-from couchdbkit.resource import ResourceNotFound
-from couchdbkit.client import Server, Database
-from couchdbkit.schema import *
-from couchdbkit.properties import *
-from couchdbkit.properties_proxy import *
-
-from couchdbkit.session import create_session
-from couchdbkit.exceptions import *
+from couchdbkit import *
 
 class DocumentTestCase(unittest.TestCase):
     def setUp(self):
@@ -436,7 +429,6 @@ class DocumentTestCase(unittest.TestCase):
         doc1.save()
         db.save_doc(design_doc)
         results = TestDoc.view('test/all')
-        print results
         self.assert_(len(results) == 2)
         self.server.delete_db('couchdbkit_test')
         
@@ -582,18 +574,27 @@ class PropertyTestCase(unittest.TestCase):
 
     def setUp(self):
         self.server = Server()
+        try:
+            self.db = self.server.create_db('couchdbkit_test')
+        except: 
+            # waiting we fix all tests use created db
+            self.db = self.server['couchdbkit_test']
+            
+    def tearDown(self):
+        try:
+            self.server.delete_db('couchdbkit_test')
+        except:
+            pass
 
     def testRequired(self):
-        db = self.server.create_db('couchdbkit_test')
         class Test(Document):
             string = StringProperty(required=True)
-        Test._db =db
+        Test._db = self.db
 
         test = Test()
         def ftest():
             test.string = ""
         self.assertRaises(BadValueError, test.save)
-        self.server.delete_db('couchdbkit_test')
 
     def testValidator(self):
         def test_validator(value):
@@ -677,13 +678,9 @@ class PropertyTestCase(unittest.TestCase):
        
         self.assert_(isinstance(test.field, basestring))
         self.assert_(isinstance(test.field1, datetime.datetime))
-        db = self.server.create_db('couchdbkit_test')
-        Test._db = db
-        
+        Test._db = self.db
         test.save()
-
         doc2 = Test.get(test.id)
-        self.server.delete_db('couchdbkit_test')
         
         v = doc2.field
         v1 = doc2.field1
@@ -699,13 +696,11 @@ class PropertyTestCase(unittest.TestCase):
                 field1 = datetime.datetime(2008, 11, 10, 8, 0, 0),
                 dynamic_field = 'test')
 
-        db = self.server.create_db('couchdbkit_test')
-        Test._db =db
+        Test._db = self.db
 
         test.save()
 
         doc2 = Test.get(test.id)
-        self.server.delete_db('couchdbkit_test')
 
         v1 = doc2.field1
         vd = doc2.dynamic_field
@@ -727,13 +722,11 @@ class PropertyTestCase(unittest.TestCase):
         doc.schema.astring = u"test"
         self.assert_(doc.schema.astring == u"test")
         self.assert_(doc._doc['schema']['astring'] == u"test")
-        db = self.server.create_db('couchdbkit_test')
         
-        MyDoc._db = db
+        MyDoc._db = self.db
 
         doc.save()
         doc2 = MyDoc.get(doc.id)
-        self.server.delete_db('couchdbkit_test')
         
         self.assert_(isinstance(doc2.schema, MySchema) == True)
         self.assert_(doc2.schema.astring == u"test")
@@ -745,9 +738,8 @@ class PropertyTestCase(unittest.TestCase):
             class b_schema(DocumentSchema):
                 name = StringProperty(  required = True, default = "name" )
             b = SchemaProperty( b_schema )
-
-        db = self.server.create_db('couchdbkit_test')
-        B._db = db
+            
+        B._db = self.db
         
         b = B()
         self.assertEquals(b.b.name, "name" )
@@ -763,7 +755,6 @@ class PropertyTestCase(unittest.TestCase):
         except BadValueError:
             pass
         b1.b.name = u"test"
-        del self.server['couchdbkit_test']
 
 
     def testSchemaWithPythonTypes(self):
@@ -895,31 +886,83 @@ class PropertyTestCase(unittest.TestCase):
         from datetime import datetime
         class A(Document):
             l = ListProperty(datetime)
+        A.set_db(self.db) 
             
+        # we can save an empty list
         a = A()
         self.assert_(a._doc == {'doc_type': 'A', 'l': []})
+        a.save()
+        self.assert_(a['_id'])
+        self.assert_(a['l']==[])
+        
+        a = A()
+        d = datetime(2009, 4, 13, 22, 56, 10, 967388)
+        a.l.append(d)
+        self.assert_(len(a.l) == 1)
+        self.assert_(a.l[0] == datetime(2009, 4, 13, 22, 56, 10))
+        self.assert_(a._doc == {'doc_type': 'A', 'l': ['2009-04-13T22:56:10Z']})
+        a.l.append({ 's': "test"})
+        self.assert_(a.l == [datetime(2009, 4, 13, 22, 56, 10), {'s': 'test'}])
+        self.assert_(a._doc == {'doc_type': 'A', 'l': ['2009-04-13T22:56:10Z', {'s': 'test'}]}
+        )
+        
+        a.save()
+        
+        b = A.get(a.id)
+        self.assert_(len(b.l) == 2)
+        self.assert_(b.l[0] == datetime(2009, 4, 13, 22, 56, 10))
+        self.assert_(b._doc['l'] == ['2009-04-13T22:56:10Z', {'s': 'test'}])
+        
+    def testListPropertyNotEmpty(self):
+        from datetime import datetime
+        class A(Document):
+            l = ListProperty(datetime, required=True)
+
+        a = A()
+        self.assert_(a._doc == {'doc_type': 'A', 'l': []})
+        self.assertRaises(BadValueError, a.save)
+        try:
+            a.validate()
+        except BadValueError, e:
+            pass
+        self.assert_(str(e) == 'Property l is required.')
         
         d = datetime(2009, 4, 13, 22, 56, 10, 967388)
         a.l.append(d)
         self.assert_(len(a.l) == 1)
         self.assert_(a.l[0] == datetime(2009, 4, 13, 22, 56, 10))
         self.assert_(a._doc == {'doc_type': 'A', 'l': ['2009-04-13T22:56:10Z']})
+        a.validate()
         
-        db = self.server.create_db('couchdbkit_test')
-        A.set_db(db) 
-        a.save()
+        class A2(Document):
+            l = ListProperty()
+            
+        a2 = A2()            
+        self.assertTrue(a2.validate(required=False))
+        self.assertTrue(a2.validate())
+    
+    def testListPropertyWithType(self):
+        from datetime import datetime
+        class A(Document):
+            l = ListProperty(item_type=datetime)
+        a = A()    
+        a.l.append("test")
+        self.assertRaises(BadValueError, a.validate)
         
-
-        b = A.get(a.id)
-        self.assert_(len(b.l) == 1)
-        self.assert_(b.l[0] == datetime(2009, 4, 13, 22, 56, 10))
-        self.assert_(b._doc['l'] == ['2009-04-13T22:56:10Z'])
-        self.server.delete_db('couchdbkit_test')
+        class B(Document):
+            ls = StringListProperty()
+        b = B()
+        b.ls.append("test")
+        self.assertTrue(b.validate())
+        b.ls.append(datetime.utcnow())
+        self.assertRaises(BadValueError, b.validate)
         
+         
     def testDictProperty(self):
         from datetime import datetime
         class A(Document):
             d = DictProperty()
+        A.set_db(self.db)
             
         a = A()
         self.assert_(a._doc == {'d': {}, 'doc_type': 'A'})
@@ -935,6 +978,178 @@ class PropertyTestCase(unittest.TestCase):
         self.assert_(a.d['d2'] == datetime(2009, 4, 16, 16, 5, 41))
         self.assert_(a.d == {'s2': 'test', 's': 'test', 'd2': datetime(2009, 4, 16, 16, 5, 41), 'created': datetime(2009, 4, 16, 16, 5, 41)})
         
+        a = A()
+        a.d['test'] = { 'a': datetime(2009, 5, 10, 21, 19, 21, 127380) }
+        self.assert_(a.d == { 'test': {'a': datetime(2009, 5, 10, 21, 19, 21)}})
+        self.assert_(a._doc == {'d': {'test': {'a': '2009-05-10T21:19:21Z'}}, 'doc_type': 'A'} )
+        
+        a.d['test']['b'] = "essai"
+        self.assert_(a._doc == {'d': {'test': {'a': '2009-05-10T21:19:21Z', 'b': 'essai'}}, 'doc_type': 'A'})
+        
+        a.d['essai'] = "test"
+        self.assert_(a.d == {'essai': 'test',
+         'test': {'a': datetime(2009, 5, 10, 21, 19, 21),
+                  'b': 'essai'}}
+        )
+        self.assert_(a._doc == {'d': {'essai': 'test', 'test': {'a': '2009-05-10T21:19:21Z', 'b': 'essai'}},
+         'doc_type': 'A'})
+         
+        del a.d['test']['a']
+        self.assert_(a.d == {'essai': 'test', 'test': {'b': 'essai'}})
+        self.assert_(a._doc ==  {'d': {'essai': 'test', 'test': {'b': 'essai'}}, 'doc_type': 'A'})
+        
+        a.d['test']['essai'] = { "a": datetime(2009, 5, 10, 21, 21, 11) }
+        self.assert_(a.d == {'essai': 'test',
+         'test': {'b': 'essai',
+                  'essai': {'a': datetime(2009, 5, 10, 21, 21, 11)}}}
+        )
+        self.assert_(a._doc == {'d': {'essai': 'test',
+               'test': {'b': 'essai', 'essai': {'a': '2009-05-10T21:21:11Z'}}},
+         'doc_type': 'A'}
+        )
+        
+        del a.d['test']['essai']
+        self.assert_(a._doc == {'d': {'essai': 'test', 'test': {'b': 'essai'}}, 'doc_type': 'A'})
+        
+        a = A()
+        a.d['s'] = "level1"
+        a.d['d'] = {}
+        a.d['d']['s'] = "level2"
+        self.assert_(a._doc == {'d': {'d': {'s': 'level2'}, 's': 'level1'}, 'doc_type': 'A'})
+        a.save()
+        a1 = A.get(a.id)
+        a1.d['d']['s'] = "level2 edited"
+        self.assert_(a1.d['d']['s'] == "level2 edited")
+        self.assert_(a1._doc['d']['d']['s'] == "level2 edited")
+        
+    def testDictPropertyNotEmpty(self):
+        from datetime import datetime
+        class A(Document):
+            d = DictProperty(required=True)
+        A.set_db(self.db) 
 
+        a = A()
+        self.assert_(a._doc == {'doc_type': 'A', 'd': {}})
+        self.assertRaises(BadValueError, a.save)
+        try:
+            a.save()
+        except BadValueError, e:
+            pass
+        self.assert_(str(e) == 'Property d is required.')
+        
+        d = datetime(2009, 4, 13, 22, 56, 10, 967388)
+        a.d['date'] = d
+        self.assert_(a.d['date'] == datetime(2009, 4, 13, 22, 56, 10))
+        self.assert_(a._doc == {'doc_type': 'A', 'd': { 'date': '2009-04-13T22:56:10Z' }})
+        a.save()
+        
+        class A2(Document):
+            d = DictProperty()
+        a2 = A2()            
+        self.assertTrue(a2.validate(required=False))
+        self.assertTrue(a2.validate())
+        
+    def testDynamicDictProperty(self):
+        from datetime import datetime
+        class A(Document):
+            pass
+            
+        a = A()
+        a.d = {}
+        
+        a.d['test'] = { 'a': datetime(2009, 5, 10, 21, 19, 21, 127380) }
+        self.assert_(a.d == {'test': {'a': datetime(2009, 5, 10, 21, 19, 21, 127380)}})
+        self.assert_(a._doc == {'d': {'test': {'a': '2009-05-10T21:19:21Z'}}, 'doc_type': 'A'} )
+        
+        a.d['test']['b'] = "essai"
+        self.assert_(a._doc == {'d': {'test': {'a': '2009-05-10T21:19:21Z', 'b': 'essai'}}, 'doc_type': 'A'})
+        
+        a.d['essai'] = "test"
+        self.assert_(a.d == {'essai': 'test',
+         'test': {'a': datetime(2009, 5, 10, 21, 19, 21, 127380),
+                  'b': 'essai'}}
+        )
+        self.assert_(a._doc == {'d': {'essai': 'test', 'test': {'a': '2009-05-10T21:19:21Z', 'b': 'essai'}},
+         'doc_type': 'A'})
+         
+        del a.d['test']['a']
+        self.assert_(a.d == {'essai': 'test', 'test': {'b': 'essai'}})
+        self.assert_(a._doc ==  {'d': {'essai': 'test', 'test': {'b': 'essai'}}, 'doc_type': 'A'})
+        
+        a.d['test']['essai'] = { "a": datetime(2009, 5, 10, 21, 21, 11, 425782) }
+        self.assert_(a.d == {'essai': 'test',
+         'test': {'b': 'essai',
+                  'essai': {'a': datetime(2009, 5, 10, 21, 21, 11, 425782)}}}
+        )
+        self.assert_(a._doc == {'d': {'essai': 'test',
+               'test': {'b': 'essai', 'essai': {'a': '2009-05-10T21:21:11Z'}}},
+         'doc_type': 'A'}
+        )
+        
+        del a.d['test']['essai']
+        self.assert_(a._doc == {'d': {'essai': 'test', 'test': {'b': 'essai'}}, 'doc_type': 'A'})
+        
+    def testDynamicDictProperty2(self):
+        from datetime import datetime
+        class A(Document):
+            pass
+        
+        A.set_db(self.db)
+        
+        a = A()
+        a.s = "test"
+        a.d = {}
+        a.d['s'] = "level1"
+        a.d['d'] = {}
+        a.d['d']['s'] = "level2"
+        self.assert_(a._doc == {'d': {'d': {'s': 'level2'}, 's': 'level1'}, 'doc_type': 'A', 's': u'test'})
+        a.save()
+        
+        a1 = A.get(a.id)
+        a1.d['d']['s'] = "level2 edited"
+        self.assert_(a1.d['d']['s'] == "level2 edited")
+
+        self.assert_(a1._doc['d']['d']['s'] == "level2 edited")
+        
+    def testDynamicListProperty(self):
+        from datetime import datetime
+        class A(Document):
+            pass
+        
+        A.set_db(self.db)
+        
+        a = A()
+        a.l = []
+        a.l.append(1)
+        a.l.append(datetime(2009, 5, 12, 13, 35, 9, 425701))
+        a.l.append({ 's': "test"})
+        self.assert_(a.l == [1, datetime(2009, 5, 12, 13, 35, 9, 425701), {'s': 'test'}])
+        self.assert_(a._doc == {'doc_type': 'A', 'l': [1, '2009-05-12T13:35:09Z', {'s': 'test'}]}
+        )
+        a.l[2]['date'] = datetime(2009, 5, 12, 13, 35, 9, 425701)
+        self.assert_(a._doc == {'doc_type': 'A',
+         'l': [1,
+               '2009-05-12T13:35:09Z',
+               {'date': '2009-05-12T13:35:09Z', 's': 'test'}]}
+        )
+        a.save()
+        
+        a1 = A.get(a.id)
+        self.assert_(a1.l == [1,
+         datetime(2009, 5, 12, 13, 35, 9),
+         {u'date': datetime(2009, 5, 12, 13, 35, 9), u's': u'test'}]
+        )
+        
+        a.l[2]['s'] = 'test edited'
+        self.assert_(a.l == [1,
+         datetime(2009, 5, 12, 13, 35, 9, 425701),
+         {'date': datetime(2009, 5, 12, 13, 35, 9, 425701),
+          's': 'test edited'}]
+        )
+        self.assert_(a._doc['l'] == [1,
+         '2009-05-12T13:35:09Z',
+         {'date': '2009-05-12T13:35:09Z', 's': 'test edited'}]
+        )
+        
 if __name__ == '__main__':
     unittest.main()
